/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* cis_base.h

   Connected induced subgraphs
   Ambros Marzetta

   $Id: cis_base.h 5167 2004-10-20 16:20:33Z bremner $
 */

#include "z_base.h"
#include "z_adjlist.h"
#include "z_ps2d.h"
#include "z_rs.h"

#define cis_pointT	uint16	/* number of points */
#define cis_edgeT	adjpointT /* number of edges */
#define MAXDEGREE	1000	/* degree of graph, used for static arrays */

extern int cis_NodeTag, cis_GlobalTag;

typedef struct cis_Node {
  Z_NODE_HEADER;		/* len; tag = cis_NodeTag */
  cis_pointT numAct;		/* number of points in induced subgraph */
  boolean pt[1];		/* variable size: numPt */
} cis_Node, *cis_NodeP;

#define is_root_node(x) (x->numAct == 0)

typedef struct cis_Global {
  Z_NODE_HEADER;		/* len; tag = cis_GlobalTag */
  z_adjlistP cis_adjlist;	/* adjacency lists */
  z_ps2dP cis_coord;		/* coordinates */
  cis_pointT numPt;		/* number of points */
} cis_Global, *cis_GlobalP;

#define cis_adjPt(gp,pt)	(adjlist_adjPt(gp->cis_adjlist, pt))
				/* adjacency list of point pt */
#define cis_coord_x(gp, i)	ps2d_coord_x(gp->cis_coord, i)
#define cis_coord_y(gp, i)	ps2d_coord_y(gp->cis_coord, i)

cis_GlobalP cis_Read(FILE *infile);
/* Read Graph from infile and allocate a global node */
/* Should test whether graph is connected. */
/* Should test for parallel edges */

cis_NodeP cis_EmptySubgraph(cis_GlobalP global);

void cis_PSHeader(cis_GlobalP global, FILE *psfile, float xsize, float ysize);

void cis_PSPicture(cis_GlobalP global, cis_NodeP node, FILE *psfile,
		   float xsize, float ysize);

boolean cis_Equal(cis_GlobalP global, cis_NodeP x, cis_NodeP y);

void cis_Print(cis_GlobalP global, cis_NodeP node);

cis_NodeP cis_f(cis_GlobalP global, cis_NodeP node);

cis_NodeP cis_adj_oracle(cis_GlobalP global, cis_NodeP node, uint32 k);

boolean cis_is_root_node(cis_GlobalP global, cis_NodeP this);

unsigned int cis_Delta(cis_GlobalP global, cis_NodeP this);

void cis_base_Init(void);
