/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* pm_both.c
   Enumeration of polyominoes. Common routines for reverse search and
   backtrack implementation.

   Ambros Marzetta
   
 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:31  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.1  1998/06/17  18:17:42  marzetta
 * Initial revision
 *
 */

#include "pm_both.h"
#include <stdlib.h>

static int compare(const void *a, const void *b)
     /* Integer comparison, used by qsort */
{
  return *(int *)a - *(int *)b;
} /* compare */

boolean lexmax(int a[MAXN], int count, int xmax, int ymax)
{
  /* Return true if a[0..count-1] is lexicographically maximal compared to
     symmetric polyominoes. */
  int i, j, b[MAXN];

  qsort(a, count, sizeof(*a), compare);	/* sort points */

  for (i=count-1; i>=0; --i) b[i] = a[i];
  for (j = 0; j < 3; j++) {	/* rotate 3 times by 90 degrees */
    for (i=count-1; i>=0; --i)
      b[i] = ((j%2?xmax:ymax)-b[i]%MAXN) * MAXN + b[i]/MAXN;
    qsort(b, count, sizeof(*b), compare);
    for (i=count-1; i>=0; --i)
      if (b[i] > a[i]) return 0; /* return 0 if lexicographically greater */
      else if (b[i] < a[i]) break;
  }

  for (i=count-1; i>=0; --i) b[i] = a[i] / MAXN * MAXN + ymax-a[i] % MAXN;
				/* mirror */
  qsort(b, count, sizeof(*b), compare);
  for (i=count-1; i>=0; --i)
    if (b[i] > a[i]) return 0; /* return 0 if lexicographically greater */
    else if (b[i] < a[i]) break;
  
  for (j = 0; j < 3; j++) {	/* rotate 3 times by 90 degrees */
    for (i=count-1; i>=0; --i)
      b[i] = ((j%2?xmax:ymax)-b[i]%MAXN) * MAXN + b[i]/MAXN;
    qsort(b, count, sizeof(*b), compare);
    for (i=count-1; i>=0; --i)
      if (b[i] > a[i]) return 0; /* return 0 if lexicographically greater */
      else if (b[i] < a[i]) break;
  }

  return 1;
} /* lexmax */
