/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* s_seq.h (sequence, implemented as doubly linked list or as array)

   Ambros Marzetta
   
 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:31  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.1  1996/04/11  16:00:08  marzetta
 * Initial revision
 *
 */

#ifndef _S_SEQ_H_
#define _S_SEQ_H_
#include "z_base.h"		/* z_PtrProc */

/* Types ------------------------------------------------------------- */

typedef enum {s_seq_doublylinked, s_seq_numtypes} s_seq_typeT;
				/* different implementations */
#define s_seq_default s_seq_doublylinked	/* default implementation */

typedef struct s_seqT *s_seqP;	/* sequence */

typedef struct s_seq_refT *s_seq_refP; /* reference into a sequence */

#ifndef _S_SEQ_C_
typedef void *s_seq_elemP;	/* element of sequence */
#else /* _S_SEQ_C_ */
typedef struct s_seq_elemT *s_seq_elemP;	/* element of sequence */
#endif /* _S_SEQ_C_ */

/* Procedures -------------------------------------------------------- */

s_seqP s_seq_New(s_seq_typeT type);
/* Allocate an empty list. Choose an implementation
   of type type. */

void s_seq_Free(s_seqP l);
/* Deallocate the sequence l, which must be empty. */

s_seq_elemP s_seq_GetObject(s_seqP l, s_seq_refP where);
/* Retrieve the object referenced by where. */

s_seq_refP s_seq_InsertFirst(s_seqP l, s_seq_elemP x);
/* Insert x into the sequence l (at the beginning). */

s_seq_refP s_seq_InsertLast(s_seqP l, s_seq_elemP x);
/* Insert x into the sequence l (at the end). */

s_seq_refP s_seq_InsertAfter(s_seqP l, s_seq_refP r, s_seq_elemP x);
/* Insert x into the sequence l after r. */

long s_seq_numElements(s_seqP l);
/* Return the number of elements in l. */

s_seq_refP s_seq_First(s_seqP l);
/* Return a reference to the first element of the sequence.
   The sequence must not be empty. */

s_seq_refP s_seq_Last(s_seqP l);
/* Return a reference to the last element of the sequence.
   The sequence must not be empty. */

void s_seq_Delete(s_seqP l, s_seq_refP ref);
/* Delete the element referenced by ref. */

void s_seq_ForAll(s_seqP l, z_PtrProc whatToDo);
/* Call whatToDo for all elements of the sequence. WhatToDo must not
   access the sequence. */

#endif /* _S_SEQ_H_ */
