/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* z_adjlist.h

   ZRAM: Adjacency lists

   Ambros Marzetta

   95-07-14
 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:32  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.1  1996/06/03  19:00:59  marzetta
 * Initial revision
 *
 */

#ifndef _Z_ADJLIST_H_
#define _Z_ADJLIST_H_
#include "z_base.h"
#include <stdio.h>

typedef uint16 adjpointT;
#define ADJ_LAST	65535

extern int z_adjlistTag;

typedef struct z_adjlist {
  Z_NODE_HEADER;		/* len; tag = z_adjlistTag; children = 0 */
  int numPt;			/* number of points */
  int numEdge;			/* number of edges */
  adjpointT adjlist_data[1];	/* variable size */
				/* adjlist_first[]: index into adjacency list*/
				/* adjlist_adj[]: adjacency lists */
} z_adjlistT, *z_adjlistP;

#define adjlist_first(gp)	(assert(gp->tag == z_adjlistTag), \
				 gp->adjlist_data)
				/* (adjpointT *) index into adjacency list */
#define adjlist_adj(gp)		(gp->adjlist_data + gp->numPt)
				/* (adjpointT *)
				   adjacency lists, terminated by ADJ_LAST */
#define adjlist_adjPt(gp,pt)	(adjlist_adj(gp) + adjlist_first(gp)[pt])
				/* (adjpointT *) adjacency list of point pt */

z_adjlistP z_adjlist_New(int numPt, int numEdge);
/* Allocate an adjacency list with numPt points and numEdge edges */

void z_adjlist_Free(z_adjlistP node);
/* Deallocate an adjacency list */

void z_adjlist_ReadStart(z_adjlistP adjlist,
			 int *i,
			 int *index);
/* Initialize adjlist, i and index before the first call to z_adjlist_Read */

void z_adjlist_ReadEnd(z_adjlistP adjlist,
		       int *i,
		       int *index);
/* Check the consistency of the adjacency list just read */

void z_adjlist_Read(FILE *infile,
		    z_adjlistP adjlist,
		    int *i,
		    int *index,
		    int maxdegree);
/* Read adjacency list of point i. Does not test for parallel edges.
   Abort if the point's degree is > maxdegree.
   */

void z_adjlist_Init(void);
/* Initialize this module */

#endif /* _Z_ADJLIST_H_ */
