/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* z_bt.h (ZRAM: Backtracking)
   Ambros Marzetta

 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.4  2004/10/20 16:20:32  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.3  2001/08/17 13:24:41  bremner
 * checkpointing not obviously broken
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.1  1998/03/10  15:03:35  marzetta
 * Initial revision
 *
 */

#ifndef _Z_BT_H_
#define _Z_BT_H_
#include "z_base.h"		/* z_NodeP */
#include "z_ps.h"		/* procedure sets */

typedef z_NodeP z_bt_GlobalP;	/* application global variables */

typedef void (*z_bt_DoProc)(z_NodeP np, void *ref);
typedef boolean (*z_bt_BranchProc)(z_bt_GlobalP global, z_NodeP np,
				   z_bt_DoProc whatToDo, void *ref);
/* The BranchInPlace function must return false (true is reserved for
   future extensions). */

typedef struct z_bt_ProcSet {
  z_bt_BranchProc		bt_BranchInPlace;
} z_bt_ProcSetT, *z_bt_ProcSetP;

/* Procedures ---------------------------------------------------------- */
/*   All z_bt_Enumerate* procedures return NULL. */

/* z_bt1.c ------------------------------------------------------------- */

void z_bt1_Init(void);
/* Initialize this module */

/* Sequential recursive depth-first enumeration */

z_NodeP z_bt_EnumerateDepthFirst(z_ProcSetNum procsetnum,
				 z_bt_GlobalP global,
				 z_NodeP root);

/* z_bt2.c ------------------------------------------------------------- */

void z_bt2_Init(void);
/* Initialize this module */

/* Sequential or parallel, using the load balancer and checkpointing */

z_NodeP z_bt_EnumerateDFExp(z_ProcSetNum procsetnum,
			    z_bt_GlobalP global,
			    z_NodeP root,
			    boolean sequential);

z_NodeP z_bt_RestartDFExp(z_ProcSetNum procsetnum,
			  z_bt_GlobalP global,
			  boolean sequential);
#endif /* _Z_BT_H_ */
