/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* z_par_intern.h (parallel base functions, not for user)

   Ambros Marzetta

   This file is included by z_main.c and the machine dependent
   files z_md_*.c.
   
 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:32  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.3  1995/08/22  12:39:45  marzetta
 * New message passing, MPI
 *
 * Revision 1.2  1995/07/28  14:05:58  marzetta
 * Some cosmetic changes
 *
 * Revision 1.1  1995/07/27  11:39:06  marzetta
 * Initial revision
 *
 */

#ifndef _Z_PAR_INTERN_H_
#define _Z_PAR_INTERN_H_
#include "z_par.h"

extern struct buffer *inbuffer, *outbuffer;
extern long inpos, outpos;
extern boolean receiving;	/* should be removed #ifdef NDEBUG */

void z_pe_switch(void);
/* (PE only) Process the message in inbuffer, i.e. executed an installed
   job which must deposit its result in outbuffer. Send the results to
   the front end.
   Called by z_pe_loop, z_md_WaitWork. */

void z_pe_loop(void);
/* (PE only) Repeatedly call z_pe_switch until a terminate job is executed
   by z_par_Fin on the front end.
   Called by z_main and MusicSlave().
   */

/* Initialization --------------------------------------------------- */

void z_par_Init(int myproc, int numprocs, boolean frontend);
/* Initialize this module (z_par.o). Called by z_md_Init and MusicInitNode.
   Pre: inbuffer and outbuffer are allocated */

void z_par_Fin(void);
/* FE: Terminate other processes.
   Call z_md_Fin.
   Called by z_main.
   */

#endif /* _Z_PAR_INTERN_H_ */
