/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* z_spq.h (speculative priority queues)

   Ambros Marzetta

 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:33  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.3  1996/04/09  13:23:25  marzetta
 * Added SEQUENTIAL and some comments
 *
 */

#ifndef _Z_SPQ_H_
#define _Z_SPQ_H_
#include "s_pq.h"

/* Types ------------------------------------------------------------- */

typedef struct z_spqT *z_spqP;	/* speculative priority queue */

/* Procedures -------------------------------------------------------- */

z_spqP z_spq_NewSpeculativePriorityQueue(z_pq_CompProcP compare);
/* Allocate an empty speculative priority queue. This procedure must be
   called on all processors at the same time. */

void z_spq_FreeSpeculativePriorityQueue(z_spqP pq);
/* Deallocate the speculative priority queue pq, which must be empty.
   This procedure must be called on all processors at the same time. */

void z_spq_Insert(z_spqP pq, z_NodeP x);
/* Insert x into the speculative priority queue. */

z_NodeP z_spq_GetDeleteNearMinimum(z_spqP pq);
/* Retrieve an element near the minimum and delete it. Return NULL if the
   speculative queue is empty (this uses the termination detection algorithm
   from z_mp.h). */

long z_spq_numLocalElements(z_spqP pq);
/* Return the number of elements in the local part of pq. */

void z_spq_DeleteLocalAbove(z_spqP pq, z_NodeP upperBound,
			    whatToDoProcP whatToDo);
/* Remove all elements >= upperBound from the priority queue. Call
   whatToDo for each of them. WhatToDo must not access the priority
   queue, but should e.g. deallocate the elements. */

char *z_spq_InfoString(z_spqP pq);
/* Return statistical information about the queue. This is a static string. */

void z_spq_init(void);

/* Elements of the speculative priority queue must be z_NodeP. Their size
   is used to transfer them to other processors. They are deallocated by
   z_FreeNode().

   The speculative priority queue uses the termination detection algorithm
   from z_mp.h. The application must call z_StartMessagePassing() before
   the first operation on the spq.
   
   In the current implementation, only one speculative priority queue may
   be allocated at the same time.
   */

#endif /* _Z_SPQ_H_ */

